/*
 * [PrepArizona.java]
 *
 * Summary: One shot program to process tax data for Arizona. Generates code for AmericanTax.java table.
 *
 * Copyright: (c) 2008-2017 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.8+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2008-06-01
 */
package com.mindprod.americantax;

import com.mindprod.csv.CSVReader;

import java.io.IOException;
import java.util.HashMap;

/**
 * One shot program to process tax data for Arizona. Generates code for AmericanTax.java table.
 * <p/>
 * Generates code for AmericanTax.java table.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2008-06-01
 * @since 2008-06-01
 */
public final class PrepArizona extends PrepStateBase
    {
    /*
     * for Arizona:  Last updated 2013-03-29
     *
     * overview: in html http://phoenix.about.com/library/blsalestaxrates.htm, slightly out of date.
     * download http://www.azdor.gov/Portals/0/TPTRates/201303.pdf  arizonaraw.pdf
     * download http://www.arizonatax.org/sites/default/files/publications/sales_tax_rates/retail_class-august_2012_0
     * .pdf arizonaraw2.pdf
     * arizonaraw.pdf contains 5 tables..
     * Sales tax is called TPT transaction privilege tax. There is a city and county sales tax.
     * Save as text, strip out all but the last column which includes state tax.
     * You need to manually create a arizonacounties.csv file by eyeballing arizonaraw.pdf like this:
     * arizonacounties.csv
     * #code, county,    county tax (including state)
     * APA, Apache,       7.1
     * COC, Coconino,   7.725
     * COH, Cochise,      7.1
     * GLA, Gila,         7.6
     *
     * then extract records from an exported arizonaraw2.pdf PDF, and manually apply county codes
     * and strip off country headers.
     * arizonacities.csv
     * # county, city, city rate( including city, county and state)
     * APA, Eagar,             10.10
     * APA, Springerville,     10.10
     * APA, St. Johns,         10.10
     * COC, Flagstaff,         9.446
     * COC, Fredonia,         11.725
     * COC, Page,             10.725
     * COC, Sedona,           10.725
     * COC, Tusayan,           9.725
     * COC, Williams,         10.725
     * configure state tax in statetax.csv and PrepArizona constructor.
     * PrepArizona will leave results in arizonadistricts.csv
     *
     */
    private final HashMap<String, String> countiesByAbbr = new HashMap<>( 50 );

    /**
     * Constructor
     */
    private PrepArizona()
        {
        super( "AZ",
                "arizona",
                6.6,   // used to strip state tax from figures. must also be recored in statetax.csv
                true /* counties */,
                true /* cities */,
                true /* city and county files include state rate */,
                false /* no convert to book case */,
                /* guess country rates */
                150 );
        }

    /**
     * Default method to read and prepare one county record.
     * You must read, and add SalesTaxItem to salesTaxItems, and skipToNewLine
     *
     * @param c open CSV reader.
     *
     * @throws IOException if cannot read file
     */
    void prepareCity( final CSVReader c ) throws IOException
        {
        final String countyAbbr = c.get();
        final String city = c.get();
        final double percent = c.getDouble() - stateTax; // strip state tax.
        c.skipToNextLine();
        final String county = countiesByAbbr.get( countyAbbr );
        assert county != null : "Undefined county:" + countyAbbr;
        salesTaxItems.add( new SalesTaxItem( county, city, percent ) );
        }

    /**
     * Default method to read and prepare one county record.
     * You must read, and add SalesTaxItem to salesTaxItems, and skipToNewLine
     *
     * @param c open CSV reader.
     *
     * @throws IOException if cannot read file
     */
    void prepareCounty( final CSVReader c ) throws IOException
        {
        // APA, Apache
        final String countyAbbr = c.get();
        final String county = c.get();
        final double percent = c.getDouble() - stateTax;   // strip state tax
        c.skipToNextLine();
        countiesByAbbr.put( countyAbbr, county );
        salesTaxItems.add( new SalesTaxItem( county, "", percent ) );
        }

    @SuppressWarnings( { "InfiniteLoopStatement" } )
    public static void main( String[] args ) throws IOException
        {
        new PrepArizona().prepare();
        }
    }
